@@ -1,5 +1,4 @@
 TAopt <- function(OF, algo = list(), ...) {
-
     algoD <- list(nD = 2000L, ## random steps for computing thresholds
                   nT = 10L,   ## number of thresholds
                   nS = 1000L, ## steps per threshold
@@ -24,8 +23,8 @@ TAopt <- function(OF, algo = list(), ...) {
         stop("'algo$neighbour' must be a function")
     if (is.null(algoD$x0))
         stop("specify start solution 'algo$x0'")
-    if (is.function(algoD$x0)) ### evaluate x0 if function
-        x0 <- algoD$x0() else x0 <- algoD$x0
+    if (is.function(algoD$x0)) ## evaluate x0 if function
+        x0 <- algoD$x0() else x0 <- eval(algoD$x0)
 
     OF1 <- function(x) OF(x, ...)
     N1 <- function(x) algoD$neighbour(x, ...)
@@ -45,7 +44,6 @@ TAopt <- function(OF, algo = list(), ...) {
     niter <- nS * nT * (stepUp+1L)
 
 
-
     ## compute thresholds
     if (is.null(algoD$vT)) {
         if (algoD$q < 1e-13) {
@@ -53,6 +51,7 @@ TAopt <- function(OF, algo = list(), ...) {
         } else {
             if (printDetail) {
                 cat("\nComputing thresholds ... ")
+                flush.console()
                 gc(FALSE)
                 startTime <- proc.time()
             }
@@ -138,7 +137,7 @@ TAopt <- function(OF, algo = list(), ...) {
             ## number of iterations
             counter <- counter + 1L
 
-            if(printBar)
+            if (printBar)
                 setTxtProgressBar(whatGen, value = counter)
 
             ## store OF values
@@ -150,7 +149,7 @@ TAopt <- function(OF, algo = list(), ...) {
             ## store solutions
             if (algoD$storeSolutions) {
                 xlist[[c(1L, counter)]] <- xn
-                xlist[[c(1L, counter)]] <- xc
+                xlist[[c(2L, counter)]] <- xc
             }
 
             ## print info
