@@ -1,8 +1,8 @@
 ## -*- truncate-lines: t; -*-
-
-require("NMOF")
-
-## optimisation functions, MA, testFunctions, option pricing
+## MA
+## option pricing
+## testFunctions
+## restartOpt
 
 ## MA
 test.MA <- function() {
@@ -21,10 +21,12 @@ test.MA <- function() {
 
 ## HESTON
 test.callHestoncf <- function() {
-    S <- 100; X <- 100; tau <- 1; r <- 0.02; q <- 0.01; v0 <- 0.2^2; vT <- 0.2^2
+    S <- 100; X <- 100; tau <- 1; r <- 0.02; q <- 0.01;
+    v0 <- 0.2^2; vT <- 0.2^2
     rho <- -0.5; k <- 0.5; sigma <- 0.5
     result <- callHestoncf(S = S, X = X, tau = tau, r = r, q = q,
-                           v0 = v0, vT = vT, rho = rho, k = k, sigma = sigma)
+                           v0 = v0, vT = vT, rho = rho, k = k,
+                           sigma = sigma)
     checkEquals(round(result[[1L]], 3), 7.119)
 
     S <- 100; X <- 100; tau <- 1; r <- 0.02; q <- 0.01; v0 <- 0.2^2; vT <- 0.2^2
@@ -108,147 +110,6 @@ test.EuropeanCallBE <- function() {
     checkEquals(res,res2)
 }
 
-
-## PSopt
-test.PSopt <- function() {
-    ## test function: PS should find minimum
-    OF <- tfRosenbrock
-    size <- 3L ### define dimension
-    algo <- list(printBar = FALSE,
-                 printDetail = FALSE,
-                 nP = 50L, nG = 1000L,
-                 c1 = 0.0, c2 = 1.5,
-                 iner = 0.8, initV = 0.50, maxV = 50,
-                 min = rep(-50, size),
-                 max = rep( 50, size))
-
-    sol <- PSopt(OF = OF, algo = algo)
-    checkEquals(sol$OFvalue, 0)
-
-    ## exception: wrong size of initP
-    algo$initP <- array(0, dim = c(20L,20L))
-    checkException(res <- PSopt(OF = OF, algo), silent = TRUE)
-    algo$initP <- function() array(0, dim = c(5,20))
-    checkException(res <- PSopt(OF = OF, algo), silent = TRUE)
-    algo$initP <- NULL
-
-
-    ## check if Fmat/xlist are returned
-    ## ...if FALSE
-    algo <- list(printBar = FALSE,
-                 printDetail = FALSE,
-                 nP = 50L, nG = 100L,
-                 c1 = 0.0, c2 = 1.5,
-                 iner = 0.8, initV = 0.50, maxV = 50,
-                 min = rep(-50, size),
-                 max = rep( 50, size),
-                 storeF = FALSE,
-                 storeSolutions = FALSE)
-    sol <- PSopt(OF = OF, algo = algo)
-    checkTrue(is.na(sol$Fmat))
-    checkTrue(is.na(sol$xlist))
-    checkEquals(length(sol$Fmat), 1L)
-    checkEquals(length(sol$xlist), 1L)
-
-    ## ...if TRUE
-    algo$storeF <- TRUE
-    algo$storeSolutions <- TRUE
-    sol <- PSopt(OF = OF, algo = algo)
-    checkEquals(names(sol$xlist), c("P","Pbest"))
-    checkEquals(dim(sol$xlist[[c(1L, algo$nG)]]),
-                c(length(algo$min),algo$nP))
-    checkEquals(dim(sol$xlist[[c(2L, algo$nG)]]),
-                c(length(algo$min),algo$nP))
-    ## xlist has only two elements
-    checkException(sol$xlist[[c(3L, algo$nG)]], silent = TRUE)
-    ## xlist[[i]] stores only algo$nG elements
-    checkException(sol$xlist[[c(2L, algo$nG + 1L)]], silent = TRUE)
-}
-
-
-## TAopt
-test.TAopt <- function() {
-
-    ## TA should come close to the minimum
-    xTRUE <- runif(5)
-    data <- list(xTRUE = xTRUE, step = 0.02)
-    OF <- function(x, data) max(abs(x - data$xTRUE))
-    neighbour <- function(x, data) {
-        x <- x + runif(length(data$xTRUE))*data$step - data$step/2
-        x
-    }
-    x0 <- runif(5)
-    algo <- list(q = 0.05, nS = 1000L, nT = 15L,
-                 neighbour = neighbour, x0 = x0,
-                 printBar = FALSE,
-                 printDetail = FALSE)
-    res <- TAopt(OF, algo = algo, data = data)
-    checkTrue(res$OFvalue < 0.005)
-
-    ## length(returned thresholds) == specified length(thresholds)
-    checkTrue(length(res$vT) == algo$nT)
-
-    ## specified thresholds are used
-    algo$vT <- c(0.1,0.05,0)
-    algo$nS <- 1000L
-    res <- TAopt(OF, algo = algo, data = data)
-    checkEqualsNumeric(res$vT,algo$vT)
-
-    ## stepUp is used
-    algo$stepUp <- 2L
-    res <- TAopt(OF, algo = algo, data = data)
-    checkEqualsNumeric(res$vT, rep(algo$vT, 3L))
-
-    ## scale is used
-    algo$stepUp <- 0L
-    algo$scale <- 1.5
-    res <- TAopt(OF, algo = algo, data = data)
-    checkEqualsNumeric(res$vT, algo$scale*c(0.1,0.05,0))
-
-    ## q is zero
-    algo <- list(q = 0, nS = 100L, nT = 15L,
-                 neighbour = neighbour, x0 = x0,
-                 printBar = FALSE,
-                 printDetail = FALSE)
-    res <- TAopt(OF, algo = algo, data = data)
-    checkEqualsNumeric(res$vT, numeric(algo$nT))
-    checkEquals(length(res$vT), algo$nT)
-
-    ## check printDetail
-    algo <- list(q = 0, nS = 100L, nT = 5L,
-                 neighbour = neighbour, x0 = x0,
-                 printBar = TRUE,
-                 printDetail = 50)
-    res <- capture.output(ignore <- TAopt(OF, algo = algo, data = data))
-    checkEquals(sum(grepl("Best solution", res)), 11L)
-}
-
-
-## LSopt
-test.LSopt <- function() {
-    xTRUE <- runif(5)
-    data <- list(xTRUE = xTRUE, step = 0.02)
-    OF <- function(x, data) max(abs(x - data$xTRUE))
-    neighbour <- function(x, data) {
-        x <- x + runif(length(data$xTRUE))*data$step - data$step/2
-        x
-    }
-    x0 <- runif(5)
-    algo <- list(nS = 10000L,
-                 neighbour = neighbour, x0 = x0,
-                 printBar = FALSE, printDetail = FALSE)
-    res <- LSopt(OF, algo = algo, data = data)
-    checkTrue(res$OFvalue < 0.005)
-
-    algo <- list(nS = 10000L,
-                 neighbour = neighbour, x0 = x0,
-                 printBar = FALSE, printDetail = 1000)
-    res <- LSopt(OF, algo = algo, data = data)
-    checkTrue(res$OFvalue < 0.005)
-
-}
-
-
 ## TESTfunctions
 test.testFunctions <- function() {
     x <- rep(0,10L)
@@ -268,24 +129,71 @@ test.testFunctions <- function() {
                        tolerance = 1e-4)
 }
 
-
-
 ## restartOpt
 test.restartOpt <- function() {
+    testParallel <- FALSE
+
     xTRUE <- runif(5L)
     data <- list(xTRUE = xTRUE, step = 0.02)
     OF <- function(x, data)
         max(abs(x - data$xTRUE))
-
     neighbour <- function(x, data)
         x + runif(length(data$xTRUE))*data$step - data$step/2
 
     x0 <- runif(5L)
-    algo <- list(q = 0.05, nS = 50L, nT = 5L,
+    algo <- list(q = 0.05, nS = 5L, nT = 5L,
                  neighbour = neighbour, x0 = x0,
                  printBar = FALSE, printDetail = FALSE)
 
-    sols <- restartOpt(fun = TAopt, n = 10L,
-                       OF = OF, algo = algo, data = data)
-    checkEquals(length(sols), 10L)
+        sols <- restartOpt(fun = TAopt, n = 5L,
+                           OF = OF, algo = algo, data = data)
+    checkEquals(length(sols), 5L)
+
+    ## tests for snow/multicore: slow!
+    if (testParallel) {
+        OF <- function(x, data) {
+            Sys.sleep(1e-3)
+            max(abs(x - data$xTRUE))
+        }
+        if (require("snow", quietly = TRUE)){
+            system.time({
+                sols <- restartOpt(fun = TAopt, n = 10L,
+                                   OF = OF, algo = algo, data = data,
+                                   method = "snow", cl = 2)
+            })
+            checkEquals(length(sols), 10L)
+
+            ## up top version 0.23-1, an argument passed with '...'
+            ## could not be called 'X': led to an error
+            X <- list(xTRUE = runif(5L), step = 0.02)
+            OF <- function(x, X)
+                max(abs(x - X$xTRUE))
+            neighbour <- function(x, X)
+                x + runif(length(X$xTRUE))*X$step - X$step/2
+            algo <- list(q = 0.05, nS = 10L, nT = 5L,
+                         neighbour = neighbour, x0 = runif(5),
+                         printBar = FALSE, printDetail = FALSE)
+            sols <- restartOpt(fun = TAopt, n = 4L,
+                               OF = OF, algo = algo, X = X)
+            sols <- restartOpt(fun = TAopt, n = 4L,
+                               OF = OF, algo = algo, X = X,
+                               method = "snow", cl = 2L)
+        }
+        if (suppressWarnings(require("multicore", quietly = TRUE))) {
+            ## up top version 0.23-1, an argument passed with '...'
+            ## could not be called 'X': led to an error
+            X <- list(xTRUE = runif(5L), step = 0.02)
+            OF <- function(x, X)
+                max(abs(x - X$xTRUE))
+            neighbour <- function(x, X)
+                x + runif(length(X$xTRUE))*X$step - X$step/2
+            algo <- list(q = 0.05, nS = 10L, nT = 5L,
+                         neighbour = neighbour, x0 = runif(5),
+                         printBar = FALSE, printDetail = FALSE)
+            sols <- restartOpt(fun = TAopt, n = 4L,
+                               OF = OF, algo = algo, X = X,
+                               method = "multicore")
+        }
+    }
+
 }
